from enum import Enum, auto

class ValenceTypeEnum(Enum):
    AROUSAL_ANEW = auto()
    DOMINANCE_ANEW = auto()
    AWE_GALC = auto()
    AMUSEMENT_GALC = auto()
    ANGER_GALC = auto()
    ANXIETY_GALC = auto()
    BEINGTOUCHED_GALC = auto()
    BOREDOM_GALC = auto()
    COMPASSION_GALC = auto()
    CONTEMPT_GALC = auto()
    CONTENTMENT_GALC = auto()
    DESPERATION_GALC = auto()
    DISAPPOINTMENT_GALC = auto()
    DISGUST_GALC = auto()
    DISSATISFACTION_GALC = auto()
    ENVY_GALC = auto()
    FEAR_GALC = auto()
    FEELINGLOVE_GALC = auto()
    GRATITUDE_GALC = auto()
    GUILT_GALC = auto()
    HAPPINESS_GALC = auto()
    HATRED_GALC = auto()
    HOPE_GALC = auto()
    HUMILITY_GALC = auto()
    ENTHUSIASM_GALC = auto()
    IRRITATION_GALC = auto()
    JEALOUSY_GALC = auto()
    JOY_GALC = auto()
    LONGING_GALC = auto()
    LUST_GALC = auto()
    ENJOYMENT_GALC = auto()
    PRIDE_GALC = auto()
    SERENITY_GALC = auto()
    RELIEF_GALC = auto()
    SADNESS_GALC = auto()
    SHAME_GALC = auto()
    SURPRISE_GALC = auto()
    STRESS_GALC = auto()
    POSITIVE_GALC = auto()
    NEGATIVE_GALC = auto()
    POSITIV_GI = auto()
    NEGATIV_GI = auto()
    PSTV_GI = auto()
    AFFIL_GI = auto()
    NGTV_GI = auto()
    HOSTILE_GI = auto()
    STRONG_GI = auto()
    POWER_GI = auto()
    WEAK_GI = auto()
    SUBMIT_GI = auto()
    ACTIVE_GI = auto()
    PASSIVE_GI = auto()
    PLEASUR_GI = auto()
    PAIN_GI = auto()
    FEEL_GI = auto()
    AROUSAL_GI = auto()
    EMOT_GI = auto()
    VIRTUE_GI = auto()
    VICE_GI = auto()
    OVRST_GI = auto()
    UNDRST_GI = auto()
    ACADEM_GI = auto()
    DOCTRIN_GI = auto()
    ECON_2_GI = auto()
    EXCH_GI = auto()
    ECON_GI = auto()
    EXPRSV_GI = auto()
    LEGAL_GI = auto()
    MILIT_GI = auto()
    POLIT_2_GI = auto()
    POLIT_GI = auto()
    RELIG_GI = auto()
    ROLE_GI = auto()
    COLL_GI = auto()
    WORK_GI = auto()
    RITUAL_GI = auto()
    SOCREL_GI = auto()
    RACE_GI = auto()
    KIN_2_GI = auto()
    MALE_GI = auto()
    FEMALE_GI = auto()
    NONADLT_GI = auto()
    HU_GI = auto()
    ANI_GI = auto()
    PLACE_GI = auto()
    SOCIAL_GI = auto()
    REGION_GI = auto()
    ROUTE_GI = auto()
    AQUATIC_GI = auto()
    LAND_GI = auto()
    SKY_GI = auto()
    OBJECT_GI = auto()
    TOOL_GI = auto()
    FOOD_GI = auto()
    VEHICLE_GI = auto()
    BLDGPT_GI = auto()
    COMNOBJ_GI = auto()
    NATOBJ_GI = auto()
    BODYPT_GI = auto()
    COMFORM_GI = auto()
    COM_GI = auto()
    SAY_GI = auto()
    NEED_GI = auto()
    GOAL_GI = auto()
    TRY_GI = auto()
    MEANS_GI = auto()
    PERSIST_GI = auto()
    COMPLET_GI = auto()
    FAIL_GI = auto()
    NATRPRO_GI = auto()
    BEGIN_GI = auto()
    VARY_GI = auto()
    INCREAS_GI = auto()
    DECREAS_GI = auto()
    FINISH_GI = auto()
    STAY_GI = auto()
    RISE_GI = auto()
    EXERT_GI = auto()
    FETCH_GI = auto()
    TRAVEL_GI = auto()
    FALL_GI = auto()
    THINK_GI = auto()
    KNOW_GI = auto()
    CAUSAL_GI = auto()
    OUGHT_GI = auto()
    PERCEIV_GI = auto()
    COMPARE_GI = auto()
    EVAL_2_GI = auto()
    EVAL_GI = auto()
    SOLVE_GI = auto()
    ABS_2_GI = auto()
    ABS_GI = auto()
    QUALITY_GI = auto()
    QUAN_GI = auto()
    NUMB_GI = auto()
    ORD_GI = auto()
    CARD_GI = auto()
    FREQ_GI = auto()
    DIST_GI = auto()
    TIME_2_GI = auto()
    TIME_GI = auto()
    SPACE_GI = auto()
    POS_GI = auto()
    DIM_GI = auto()
    REL_GI = auto()
    COLOR_GI = auto()
    SELF_GI = auto()
    OUR_GI = auto()
    YOU_GI = auto()
    NAME_GI = auto()
    YES_GI = auto()
    NO_GI = auto()
    NEGATE_GI = auto()
    INTRJ_GI = auto()
    IAV_GI = auto()
    DAV_GI = auto()
    SV_GI = auto()
    IPADJ_GI = auto()
    INDADJ_GI = auto()
    POSITIVE_HULIU = auto()
    NEGATIVE_HULIU = auto()
    POWGAIN_LASSWELL = auto()
    POWLOSS_LASSWELL = auto()
    POWENDS_LASSWELL = auto()
    POWAREN_LASSWELL = auto()
    POWCON_LASSWELL = auto()
    POWCOOP_LASSWELL = auto()
    POWAUPT_LASSWELL = auto()
    POWPT_LASSWELL = auto()
    POWDOCT_LASSWELL = auto()
    POWAUTH_LASSWELL = auto()
    POWOTH_LASSWELL = auto()
    POWTOT_LASSWELL = auto()
    RCETHIC_LASSWELL = auto()
    RCRELIG_LASSWELL = auto()
    RCGAIN_LASSWELL = auto()
    RCLOSS_LASSWELL = auto()
    RCENDS_LASSWELL = auto()
    RCTOT_LASSWELL = auto()
    RSPGAIN_LASSWELL = auto()
    RSPLOSS_LASSWELL = auto()
    RSPOTH_LASSWELL = auto()
    RSPTOT_LASSWELL = auto()
    AFFGAIN_LASSWELL = auto()
    AFFLOSS_LASSWELL = auto()
    AFFPT_LASSWELL = auto()
    AFFOTH_LASSWELL = auto()
    AFFTOT_LASSWELL = auto()
    WLTPT_LASSWELL = auto()
    WLTTRAN_LASSWELL = auto()
    WLTOTH_LASSWELL = auto()
    WLTTOT_LASSWELL = auto()
    WLBGAIN_LASSWELL = auto()
    WLBLOSS_LASSWELL = auto()
    WLBPHYS_LASSWELL = auto()
    WLBPSYC_LASSWELL = auto()
    WLBPT_LASSWELL = auto()
    WLBTOT_LASSWELL = auto()
    ENLGAIN_LASSWELL = auto()
    ENLLOSS_LASSWELL = auto()
    ENLENDS_LASSWELL = auto()
    ENLPT_LASSWELL = auto()
    ENLOTH_LASSWELL = auto()
    ENLTOT_LASSWELL = auto()
    SKLASTH_LASSWELL = auto()
    SKLPT_LASSWELL = auto()
    SKLOTH_LASSWELL = auto()
    SKLTOT_LASSWELL = auto()
    TRNGAIN_LASSWELL = auto()
    TRNLOSS_LASSWELL = auto()
    TRANLW_LASSWELL = auto()
    MEANSLW_LASSWELL = auto()
    ENDSLW_LASSWELL = auto()
    ARENALW_LASSWELL = auto()
    PTLW_LASSWELL = auto()
    NATION_LASSWELL = auto()
    ANOMIE_LASSWELL = auto()
    NEGAFF_LASSWELL = auto()
    POSAFF_LASSWELL = auto()
    SURELW_LASSWELL = auto()
    IF_LASSWELL = auto()
    NOTLW_LASSWELL = auto()
    TIMESPC_LASSWELL = auto()
    FORMLW_LASSWELL = auto()
    ANGER_NRC = auto()
    ANTICIPATION_NRC = auto()
    DISGUST_NRC = auto()
    FEAR_NRC = auto()
    JOY_NRC = auto()
    NEGATIVE_NRC = auto()
    POSITIVE_NRC = auto()
    SADNESS_NRC = auto()
    SURPRISE_NRC = auto()
    TRUST_NRC = auto()
    PLEASANTNESS_SENTICNET = auto()
    ATTENTION_SENTICNET = auto()
    SENSITIVITY_SENTICNET = auto()
    APTITUDE_SENTICNET = auto()
    POLARITY_SENTICNET = auto()
    STANFORD = auto()
    ANX_LIWC = auto()
    FUNCT_LIWC = auto()
    ANGER_LIWC = auto()
    PRONOUN_LIWC = auto()
    SAD_LIWC = auto()
    PPRON_LIWC = auto()
    COGMECH_LIWC = auto()
    I_LIWC = auto()
    INSIGHT_LIWC = auto()
    WE_LIWC = auto()
    CAUSE_LIWC = auto()
    YOU_LIWC = auto()
    DISCREP_LIWC = auto()
    SHEHE_LIWC = auto()
    TENTAT_LIWC = auto()
    THEY_LIWC = auto()
    CERTAIN_LIWC = auto()
    IPRON_LIWC = auto()
    INHIB_LIWC = auto()
    ARTICLE_LIWC = auto()
    INCL_LIWC = auto()
    VERBS_LIWC = auto()
    EXCL_LIWC = auto()
    AUXVB_LIWC = auto()
    PERCEPT_LIWC = auto()
    PAST_LIWC = auto()
    SEE_LIWC = auto()
    PRESENT_LIWC = auto()
    HEAR_LIWC = auto()
    FUTURE_LIWC = auto()
    FEEL_LIWC = auto()
    ADVERBS_LIWC = auto()
    PREP_LIWC = auto()
    CONJ_LIWC = auto()
    BIO_LIWC = auto()
    NEGATE_LIWC = auto()
    BODY_LIWC = auto()
    QUANT_LIWC = auto()
    HEALTH_LIWC = auto()
    NUMBERS_LIWC = auto()
    SEXUAL_LIWC = auto()
    SWEAR_LIWC = auto()
    INGEST_LIWC = auto()
    ASSENT_LIWC = auto()
    NONFLU_LIWC = auto()
    FILLER_LIWC = auto()
    WORK_LIWC = auto()
    ACHIEV_LIWC = auto()
    LEISURE_LIWC = auto()
    HOME_LIWC = auto()
    MONEY_LIWC = auto()
    RELIG_LIWC = auto()
    DEATH_LIWC = auto()
    SOCIAL_LIWC = auto()
    FAMILY_LIWC = auto()
    RELATIV_LIWC = auto()
    FRIENDS_LIWC = auto()
    MOTION_LIWC = auto()
    HUMANS_LIWC = auto()
    SPACE_LIWC = auto()
    AFFECT_LIWC = auto()
    TIME_LIWC = auto()
    POSEMO_LIWC = auto()
    NEGEMO_LIWC = auto()